@@ -11,6 +11,7 @@
 # WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 # See the License for the specific language governing permissions and
 # limitations under the License.
+import datetime
 import logging
 import math
 import os
@@ -33,7 +34,7 @@ from toil.common import SYS_MAX_SIZE, Config, Toil, fC
 from toil.job import JobDescription
 from toil.lib.threading import cpu_count
 
-log = logging.getLogger(__name__)
+logger = logging.getLogger(__name__)
 
 
 class SingleMachineBatchSystem(BatchSystemSupport):
@@ -84,12 +85,12 @@ class SingleMachineBatchSystem(BatchSystemSupport):
         if maxCores > self.numCores:
             if maxCores != SYS_MAX_SIZE:
                 # We have an actually specified limit and not the default
-                log.warning('Not enough cores! User limited to %i but we only have %i.', maxCores, self.numCores)
+                logger.warning('Not enough cores! User limited to %i but we only have %i.', maxCores, self.numCores)
             maxCores = self.numCores
         if maxMemory > self.physicalMemory:
             if maxMemory != SYS_MAX_SIZE:
                 # We have an actually specified limit and not the default
-                log.warning('Not enough memory! User limited to %i bytes but we only have %i bytes.', maxMemory, self.physicalMemory)
+                logger.warning('Not enough memory! User limited to %i bytes but we only have %i bytes.', maxMemory, self.physicalMemory)
             maxMemory = self.physicalMemory
 
         workdir = Toil.getLocalWorkflowDir(config.workflowID, config.workDir)  # config.workDir may be None; this sets a real directory
@@ -97,7 +98,7 @@ class SingleMachineBatchSystem(BatchSystemSupport):
         if maxDisk > self.physicalDisk:
             if maxDisk != SYS_MAX_SIZE:
                 # We have an actually specified limit and not the default
-                log.warning('Not enough disk space! User limited to %i bytes but we only have %i bytes.', maxDisk, self.physicalDisk)
+                logger.warning('Not enough disk space! User limited to %i bytes but we only have %i bytes.', maxDisk, self.physicalDisk)
             maxDisk = self.physicalDisk
 
         super().__init__(config, maxCores, maxMemory, maxDisk)
@@ -161,11 +162,11 @@ class SingleMachineBatchSystem(BatchSystemSupport):
         self.daddyException: Optional[Exception] = None
 
         if self.debugWorker:
-            log.debug('Started batch system %s in worker debug mode.', id(self))
+            logger.debug('Started batch system %s in worker debug mode.', id(self))
         else:
             self.daddyThread = Thread(target=self.daddy, daemon=True)
             self.daddyThread.start()
-            log.debug('Started batch system %s in normal mode.', id(self))
+            logger.debug('Started batch system %s in normal mode.', id(self))
 
     def daddy(self):
         """
@@ -183,7 +184,7 @@ class SingleMachineBatchSystem(BatchSystemSupport):
         """
 
         try:
-            log.debug('Started daddy thread for batch system %s.', id(self))
+            logger.debug('Started daddy thread for batch system %s.', id(self))
 
             while not self.shuttingDown.is_set():
                 # Main loop
@@ -230,16 +231,16 @@ class SingleMachineBatchSystem(BatchSystemSupport):
 
 
             # When we get here, we are shutting down.
-            log.debug('Daddy thread cleaning up %d remaining children for batch system %s...', len(self.children), id(self))
+            logger.debug('Daddy thread cleaning up %d remaining children for batch system %s...', len(self.children), id(self))
 
             self._stop_and_wait(self.children.values())
 
-            log.debug('Daddy thread for batch system %s finishing because no children should now exist', id(self))
+            logger.debug('Daddy thread for batch system %s finishing because no children should now exist', id(self))
 
             # Then exit the thread.
             return
         except Exception as e:
-            log.critical('Unhandled exception in daddy thread for batch system %s: %s', id(self), traceback.format_exc())
+            logger.critical('Unhandled exception in daddy thread for batch system %s: %s', id(self), traceback.format_exc())
             # Pass the exception back to the main thread so it can stop the next person who calls into us.
             self.daddyException = e
             raise
@@ -247,7 +248,7 @@ class SingleMachineBatchSystem(BatchSystemSupport):
     def _checkOnDaddy(self):
         if self.daddyException is not None:
             # The daddy thread broke and we cannot do our job
-            log.critical('Propagating unhandled exception in daddy thread to main thread')
+            logger.critical('Propagating unhandled exception in daddy thread to main thread')
             exc = self.daddyException
             self.daddyException = None
             if isinstance(exc, Exception):
@@ -285,7 +286,7 @@ class SingleMachineBatchSystem(BatchSystemSupport):
                 # The child process really is in its own group, and not ours.
 
                 # Kill the group, which hopefully hasn't been reused
-                log.debug('Send shutdown kill to process group %s known to batch system %s', pgid, id(self))
+                logger.debug('Send shutdown kill to process group %s known to batch system %s', pgid, id(self))
                 try:
                     os.killpg(pgid, signal.SIGKILL)
                     pgids.append(pgid)
@@ -302,10 +303,14 @@ class SingleMachineBatchSystem(BatchSystemSupport):
 
         return pgids
 
-    def _stop_and_wait(self, popens: Sequence[subprocess.Popen]) -> None:
+    def _stop_and_wait(self, popens: Sequence[subprocess.Popen], timeout: int = 5) -> None:
         """
         Stop the given child processes and all their children. Blocks until the
-        processes are gone.
+        processes are gone or timeout is passed.
+
+        :param popens: The processes to stop and wait on.
+        :param timeout: The number of seconds to wait for all process groups to
+                        be gone.
         """
 
         pgids = self._stop_now(popens)
@@ -313,27 +318,54 @@ class SingleMachineBatchSystem(BatchSystemSupport):
         for popen in popens:
             # Wait on all the children
             popen.wait()
+            logger.debug('Process %s known to batch system %s is stopped; it returned %s',
+                         popen.pid, id(self), popen.returncode)
 
-            log.debug('Process %s known to batch system %s is stopped; it returned %s', popen.pid, id(self), popen.returncode)
+        # Make sure all child processes have received their kill signal
+        self._wait_for_death(pgids, timeout)
 
-        for pgid in pgids:
-            try:
-                while True:
+    def _wait_for_death(self, pgids: List[int], timeout: int = 5):
+        """
+        Wait for the process groups to be killed. Blocks until the processes
+        are gone or timeout is passed.
+
+        :param pgids: The list of process group ids.
+        :param timeout: The number of seconds to wait for all process groups to
+                        be gone.
+        """
+        # TODO: this opens a PGID reuse risk; someone else might've reaped the
+        #  process and its PGID may have been re-used.
+
+        start = datetime.datetime.now()
+        while len(pgids) > 0 and (datetime.datetime.now() - start).total_seconds() < timeout:
+            new_pgids: List[int] = []
+            for pgid in pgids:
+                try:
                     # Send a kill to the group again, to see if anything in it
                     # is still alive. Our first kill might not have been
                     # delivered yet.
                     os.killpg(pgid, signal.SIGKILL)
-                    # If that worked it is still alive, so wait for the kernel
-                    # to stop fooling around and kill it.
-                    log.warning('Sent redundant shutdown kill to surviving process group %s known to batch system %s', pgid, id(self))
-                    time.sleep(0.1)
-            except ProcessLookupError:
-                # The group is actually gone now.
-                pass
-            except PermissionError:
-                # The group is not only gone but reused
-                pass
 
+                    # If we reach here, something in the process group still
+                    # exists.
+                    new_pgids.append(pgid)
+                except ProcessLookupError:
+                    # The group is actually gone now.
+                    pass
+                except PermissionError:
+                    # The group is not only gone but reused
+                    pass
+
+            pgids = new_pgids
+            if len(pgids) > 0:
+                time.sleep(0.1)
+
+        if len(pgids) > 0:
+            # If any processes are still alive, let user know that we may leave
+            # behind dead but unreaped processes.
+            logger.warning('Processes were not reaped in groups: %s.', str(pgids))
+            logger.warning('Make sure your jobs are cleaning up child processes appropriately to avoid zombie '
+                           'processes possibly being left behind.')
 
     def _pollForDoneChildrenIn(self, pid_to_popen):
         """
@@ -386,7 +418,7 @@ class SingleMachineBatchSystem(BatchSystemSupport):
                 if popen.poll() is not None:
                     # Process is done
                     ready.add(pid)
-                    log.debug('Child %d has stopped', pid)
+                    logger.debug('Child %d has stopped', pid)
 
             # Return all the done processes we found
             return ready
@@ -479,7 +511,7 @@ class SingleMachineBatchSystem(BatchSystemSupport):
                         self.memory.release(jobMemory)
                         self.disk.release(jobDisk)
 
-                        log.error('Could not start job %s: %s', jobID, traceback.format_exc())
+                        logger.error('Could not start job %s: %s', jobID, traceback.format_exc())
 
                         # Report as failed.
                         self.outputQueue.put(UpdatedBatchJobInfo(jobID=jobID, exitStatus=EXIT_STATUS_UNAVAILABLE_VALUE, wallTime=0, exitReason=None))
@@ -500,7 +532,7 @@ class SingleMachineBatchSystem(BatchSystemSupport):
                         info = Info(startTime, popen, (coreFractions, jobMemory, jobDisk), killIntended=False)
                         self.runningJobs[jobID] = info
 
-                        log.debug('Launched job %s as child %d', jobID, popen.pid)
+                        logger.debug('Launched job %s as child %d', jobID, popen.pid)
 
                         # Report success starting the job
                         # Note that if a PID were somehow 0 it would look like False
@@ -550,7 +582,7 @@ class SingleMachineBatchSystem(BatchSystemSupport):
             # Before we reap it (if possible), kill its PID as a PGID to make sure
             # it isn't leaving children behind.
             # TODO: This is a PGID re-use risk on Mac because the process is
-            # reaped already and the PGID may have been reused.
+            #  reaped already and the PGID may have been reused.
             try:
                 os.killpg(pid, signal.SIGKILL)
             except ProcessLookupError:
@@ -563,44 +595,23 @@ class SingleMachineBatchSystem(BatchSystemSupport):
         # See how the child did, and reap it.
         statusCode = popen.wait()
         if statusCode != 0 and not info.killIntended:
-            log.error("Got exit code %i (indicating failure) "
+            logger.error("Got exit code %i (indicating failure) "
                       "from job %s.", statusCode, self.jobs[jobID])
         if not info.killIntended:
             # Report if the job failed and we didn't kill it.
             # If we killed it then it shouldn't show up in the queue.
             self.outputQueue.put(UpdatedBatchJobInfo(jobID=jobID, exitStatus=statusCode, wallTime=time.time() - info.time, exitReason=None))
 
-        # Make absolutely sure all processes in the group have received their
-        # kill signals and been cleaned up.
-        # TODO: this opens a PGID reuse risk; we reaped the process and its
-        # PGID may have been re-used. But it probably hasn't been and we
-        # definitely want to make sure all its children died before saying the
-        # job is done. Some might not be dead yet if we don't do this.
-        # TODO: can we safely do this before reaping? Or would we sit forever
-        # signaling a dead but unreaped process?
-        try:
-            while True:
-                # Send a kill to the group again, to see if anything in it
-                # is still alive. Our first kill might not have been
-                # delivered yet.
-                os.killpg(pid, signal.SIGKILL)
-                # If that worked it is still alive, so wait for the kernel
-                # to stop fooling around and kill it.
-                log.warning('Sent redundant job completion kill to surviving process group %s known to batch system %s', pid, id(self))
-                time.sleep(0.1)
-        except ProcessLookupError:
-            # It is dead already
-            pass
-        except PermissionError:
-            # It isn't ours actually. Ours is dead.
-            pass
+        # Last attempt to make sure all processes in the group have received
+        # their kill signals.
+        self._wait_for_death([pid])
 
         # Free up the job's resources.
         self.coreFractions.release(coreFractions)
         self.memory.release(jobMemory)
         self.disk.release(jobDisk)
 
-        log.debug('Child %d for job %s succeeded', pid, jobID)
+        logger.debug('Child %d for job %s succeeded', pid, jobID)
 
     def issueBatchJob(self, jobDesc: JobDescription, job_environment: Optional[Dict[str, str]] = None) -> int:
         """Adds the command and resources to a queue to be run."""
@@ -615,7 +626,7 @@ class SingleMachineBatchSystem(BatchSystemSupport):
         # The abstract batch system can handle it.
         self.checkResourceRequest(jobDesc.memory, cores, jobDesc.disk, job_name=jobDesc.jobName,
                                   detail=f'Scale is set to {self.scale}.')
-        log.debug(f"Issuing the command: {jobDesc.command} with "
+        logger.debug(f"Issuing the command: {jobDesc.command} with "
                   f"memory: {jobDesc.memory}, cores: {cores}, disk: {jobDesc.disk}")
         with self.jobIndexLock:
             jobID = self.jobIndex
@@ -642,7 +653,7 @@ class SingleMachineBatchSystem(BatchSystemSupport):
 
         self._checkOnDaddy()
 
-        log.debug(f'Killing jobs: {jobIDs}')
+        logger.debug(f'Killing jobs: {jobIDs}')
 
         # Collect the popen handles for the jobs we have to stop
         popens: List[subprocess.Popen] = []
@@ -656,7 +667,7 @@ class SingleMachineBatchSystem(BatchSystemSupport):
                 else:
                     # No popen if running in forkless mode currently
                     assert self.debugWorker
-                    log.critical("Can't kill job: %s in debug mode" % jobID)
+                    logger.critical("Can't kill job: %s in debug mode" % jobID)
 
         # Stop them all in a batch. Don't reap, because we need the daddy
         # thread to reap them to mark the jobs as not running anymore.
@@ -701,7 +712,7 @@ class SingleMachineBatchSystem(BatchSystemSupport):
         except Empty:
             return None
         self.jobs.pop(item.jobID)
-        log.debug("Ran jobID: %s with exit value: %i", item.jobID, item.exitStatus)
+        logger.debug("Ran jobID: %s with exit value: %i", item.jobID, item.exitStatus)
         return item
 
     @classmethod
