@@ -20,9 +20,9 @@ mount the job store.
 
 Additional containers should be launched with Singularity, not Docker.
 """
-import base64
 import datetime
 import logging
+import math
 import os
 import pickle
 import time
@@ -37,6 +37,7 @@ from toil.batchSystems.abstractBatchSystem import (EXIT_STATUS_UNAVAILABLE_VALUE
                                                    BatchJobExitReason,
                                                    UpdatedBatchJobInfo)
 from toil.batchSystems.cleanup_support import BatchSystemCleanupSupport
+from toil.batchSystems.contained_executor import pack_job
 from toil.common import Config, Toil
 from toil.job import JobDescription
 from toil.lib.misc import get_public_ip, slow_down, utc_now
@@ -92,12 +93,12 @@ class TESBatchSystem(BatchSystemCleanupSupport):
             job_store_type, job_store_path = Toil.parseLocator(config.jobStore)
             if job_store_type == 'file':
                 # If we have a file job store, we want to mount it at the same path, if we can
-                self.__mount_local_path_if_possible(job_store_path, job_store_path)
+                self._mount_local_path_if_possible(job_store_path, job_store_path)
 
         # If we have AWS credentials, we want to mount them in our home directory if we can.
         aws_credentials_path = os.path.join(os.path.expanduser("~"), '.aws')
         if os.path.isdir(aws_credentials_path):
-            self.__mount_local_path_if_possible(aws_credentials_path, '/root/.aws')
+            self._mount_local_path_if_possible(aws_credentials_path, '/root/.aws')
 
         # We assign job names based on a numerical job ID. This functionality
         # is managed by the BatchSystemLocalSupport.
@@ -112,8 +113,10 @@ class TESBatchSystem(BatchSystemCleanupSupport):
         self.bs_id_to_tes_id: Dict[int, str] = {}
         self.tes_id_to_bs_id: Dict[str, int] = {}
 
-    def __server_can_mount(self, url: str) -> bool:
+    def _server_can_mount(self, url: str) -> bool:
         """
+        Internal function. Should not be called outside this class.
+
         Return true if the given URL is under a supported storage location for
         the TES server, and false otherwise.
         """
@@ -125,15 +128,17 @@ class TESBatchSystem(BatchSystemCleanupSupport):
                 return True
         return False
 
-    def __mount_local_path_if_possible(self, local_path: str, container_path: str) -> None:
+    def _mount_local_path_if_possible(self, local_path: str, container_path: str) -> None:
         """
+        Internal function. Should not be called outside this class.
+
         If a local path is somewhere the server thinks it can access, mount it
         into all the tasks.
         """
         # TODO: We aren't going to work well with linked imports if we're mounting the job store into the container...
 
         path_url = 'file://' + os.path.abspath(local_path)
-        if os.path.exists(local_path) and self.__server_can_mount(path_url):
+        if os.path.exists(local_path) and self._server_can_mount(path_url):
             # We can access this file from the server. Probably.
             self.mounts.append(tes.Input(url=path_url,
                                          path=container_path,
@@ -175,20 +180,15 @@ class TESBatchSystem(BatchSystemCleanupSupport):
             environment = self.environment.copy()
             if job_environment:
                 environment.update(job_environment)
+            if 'TOIL_WORKDIR' not in environment:
+                # The appliance container defaults TOIL_WORKDIR to
+                # /var/lib/toil, but TES doesn't (always?) give us a writable
+                # /, so we need to use the writable space in /tmp by default
+                # instead when running on TES.
+                environment['TOIL_WORKDIR'] = '/tmp'
 
-            # Make a job dict to send to the executor.
-            # TODO: Factor out executor setup from here and Kubernetes
-            job: Dict[str, Any] = {"command": job_desc.command}
-            if self.user_script is not None:
-                # If there's a user script resource be sure to send it along
-                job['userScript'] = self.user_script
-            # Encode it in a form we can send in a command-line argument. Pickle in
-            # the highest protocol to prevent mixed-Python-version workflows from
-            # trying to work. Make sure it is text so we can ship it to Kubernetes
-            # via JSON.
-            encoded_job = base64.b64encode(pickle.dumps(job, pickle.HIGHEST_PROTOCOL)).decode('utf-8')
-            # Make a command to run it in the exacutor
-            command_list = ['_toil_contained_executor', encoded_job]
+            # Make a command to run it in the executor
+            command_list = pack_job(job_desc, self.user_script)
 
             # Make the sequence of TES containers ("executors") to run.
             # We just run one which is the Toil executor to grab the user
@@ -202,8 +202,18 @@ class TESBatchSystem(BatchSystemCleanupSupport):
             task_inputs = list(self.mounts)
             # If we had any per-job input files they would come in here.
 
+            # Prepare resource requirements
+            task_resources = tes.Resources(cpu_cores=math.ceil(job_desc.cores),
+                                           ram_gb=job_desc.memory / (1024**3),
+                                           disk_gb=job_desc.disk / (1024**3),
+                                           # TODO: py-tes spells this differently than Toil
+                                           preemptible=job_desc.preemptable)
+
             # Package into a TES Task
-            task = tes.Task(name=job_name, executors=task_executors, inputs=task_inputs)
+            task = tes.Task(name=job_name,
+                            executors=task_executors,
+                            inputs=task_inputs,
+                            resources=task_resources)
 
             # Launch it and get back the TES ID that we can use to poll the task
             tes_id = self.tes.create_task(task)
@@ -216,8 +226,10 @@ class TESBatchSystem(BatchSystemCleanupSupport):
 
             return bs_id
 
-    def __get_runtime(self, task: tes.Task) -> Optional[float]:
+    def _get_runtime(self, task: tes.Task) -> Optional[float]:
         """
+        Internal function. Should not be called outside this class.
+
         Get the time that the given job ran/has been running for, in seconds,
         or None if that time is not available. Never returns 0.
         """
@@ -242,8 +254,10 @@ class TESBatchSystem(BatchSystemCleanupSupport):
         # it has been running for.
         return slow_down((end_time - start_time).total_seconds())
 
-    def __get_exit_code(self, task: tes.Task) -> int:
+    def _get_exit_code(self, task: tes.Task) -> int:
         """
+        Internal function. Should not be called outside this class.
+
         Get the exit code of the last executor with a log in the task, or
         EXIT_STATUS_UNAVAILABLE_VALUE if no executor has a log.
         """
@@ -261,6 +275,21 @@ class TESBatchSystem(BatchSystemCleanupSupport):
         # If we get here we couldn't find an exit code.
         return EXIT_STATUS_UNAVAILABLE_VALUE
 
+    def __get_log_text(self, task: tes.Task) -> Optional[str]:
+        """
+        Get the log text (standard error) of the last executor with a log in
+        the task, or None.
+        """
+
+        for task_log in reversed(task.logs or []):
+            for executor_log in reversed(task_log.logs or []):
+                if isinstance(executor_log.stderr, str):
+                    # Find the last executor log code that is a string and return it
+                    return executor_log.stderr
+
+        # If we get here we couldn't find a log.
+        return None
+
     def getUpdatedBatchJob(self, maxWait: int) -> Optional[UpdatedBatchJobInfo]:
         # Remember when we started, for respecting the timeout
         entry = datetime.datetime.now()
@@ -280,7 +309,7 @@ class TESBatchSystem(BatchSystemCleanupSupport):
                 # Immediately poll all the jobs we issued.
                 # TODO: There's no way to acknowledge a finished job, so there's no
                 # faster way to find the newly finished jobs than polling
-                task = self.tes.get_task(tes_id)
+                task = self.tes.get_task(tes_id, view="MINIMAL")
                 if task.state in ["COMPLETE", "CANCELED", "EXECUTOR_ERROR", "SYSTEM_ERROR"]:
                     # This task is done!
                     logger.debug("Found stopped task: %s", task)
@@ -294,14 +323,21 @@ class TESBatchSystem(BatchSystemCleanupSupport):
 
                     # Otherwise, it stopped running and it wasn't our fault.
 
+                    # Fetch the task's full info, including logs.
+                    task = self.tes.get_task(tes_id, view="FULL")
+
                     # Record runtime
-                    runtime = self.__get_runtime(task)
+                    runtime = self._get_runtime(task)
 
                     # Determine if it succeeded
                     exit_reason = STATE_TO_EXIT_REASON[task.state]
 
                     # Get its exit code
-                    exit_code = self.__get_exit_code(task)
+                    exit_code = self._get_exit_code(task)
+
+                    if task.state == "EXECUTOR_ERROR":
+                        # The task failed, so report executor logs.
+                        logger.warning('Log from failed executor: %s', self.__get_log_text(task))
 
                     # Compose a result
                     result = UpdatedBatchJobInfo(jobID=bs_id, exitStatus=exit_code, wallTime=runtime, exitReason=exit_reason)
@@ -331,10 +367,12 @@ class TESBatchSystem(BatchSystemCleanupSupport):
 
         for tes_id in self.tes_id_to_bs_id.keys():
             # Shut down all the TES jobs we issued.
-            self.__try_cancel(tes_id)
+            self._try_cancel(tes_id)
 
-    def __try_cancel(self, tes_id: str) -> None:
+    def _try_cancel(self, tes_id: str) -> None:
         """
+        Internal function. Should not be called outside this class.
+
         Try to cancel a TES job.
 
         Succeed if it can't be canceled because it has stopped,
@@ -363,7 +401,8 @@ class TESBatchSystem(BatchSystemCleanupSupport):
         bs_id_to_runtime = {}
 
         for tes_id, bs_id in self.tes_id_to_bs_id.items():
-            # Poll every issued task.
+            # Poll every issued task, and get the runtime info right away in
+            # the default BASIC view.
             # TODO: use list_tasks filtering by name prefix and running state!
             task = self.tes.get_task(tes_id)
             logger.debug("Observed task: %s", task)
@@ -372,7 +411,7 @@ class TESBatchSystem(BatchSystemCleanupSupport):
                 # Docker containers, and we don't want to time out on them in
                 # the tests. But they may not have any runtimes, so it might
                 # not really help.
-                runtime = self.__get_runtime(task)
+                runtime = self._get_runtime(task)
                 if runtime:
                     # We can measure a runtime
                     bs_id_to_runtime[bs_id] = runtime
@@ -386,16 +425,19 @@ class TESBatchSystem(BatchSystemCleanupSupport):
         # Kill all the ones that are local
         self.killLocalJobs(job_ids)
 
-        # TODO: implement
         for bs_id in job_ids:
             if bs_id in self.bs_id_to_tes_id:
                 # We sent this to TES. So try to cancel it.
-                self.__try_cancel(self.bs_id_to_tes_id[bs_id])
+                self._try_cancel(self.bs_id_to_tes_id[bs_id])
                 # But don't forget the mapping until we actually get the finish
                 # notification for the job.
 
+        # TODO: If the kill races the collection of a finished update, do we
+        # have to censor the finished update even if the kill never took
+        # effect??? That's not implemented.
+
     @classmethod
-    def add_tes_options(cls, parser: Union[ArgumentParser, _ArgumentGroup]) -> None:
+    def add_options(cls, parser: Union[ArgumentParser, _ArgumentGroup]) -> None:
         parser.add_argument("--tesEndpoint", dest="tes_endpoint", default=cls.get_default_tes_endpoint(),
                             help="The http(s) URL of the TES server.  (default: %(default)s)")
         parser.add_argument("--tesUser", dest="tes_user", default=None,
