@@ -23,7 +23,7 @@ import os
 import pickle
 import sys
 import time
-from typing import Dict, List, Optional, Set
+from typing import Any, Dict, List, Optional, Set, Union
 
 import enlighten
 
@@ -36,8 +36,17 @@ from toil.batchSystems.abstractBatchSystem import (
 from toil.bus import JobUpdatedMessage
 from toil.common import Config, Toil, ToilMetrics
 from toil.cwl.utils import CWL_INTERNAL_JOBS, CWL_UNSUPPORTED_REQUIREMENT_EXIT_CODE
-from toil.job import CheckpointJobDescription, JobDescription, ServiceJobDescription
-from toil.jobStores.abstractJobStore import AbstractJobStore, NoSuchJobException
+from toil.job import (
+    CheckpointJobDescription,
+    JobDescription,
+    ServiceJobDescription,
+    TemporaryID,
+)
+from toil.jobStores.abstractJobStore import (
+    AbstractJobStore,
+    NoSuchJobException,
+    NoSuchFileException
+)
 from toil.lib.conversions import bytes2human
 from toil.lib.throttle import LocalThrottle
 from toil.provisioners.abstractProvisioner import AbstractProvisioner
@@ -106,26 +115,34 @@ class FailedJobsException(Exception):
         return self.msg
 
 
-####################################################
-##Following class represents the leader
-####################################################
-
 class Leader:
-    """ Class that encapsulates the logic of the leader.
     """
-    def __init__(self, config: Config, batchSystem: AbstractBatchSystem,
-                 provisioner: AbstractProvisioner, jobStore: AbstractJobStore,
-                 rootJob: JobDescription, jobCache: Optional[Dict[str, JobDescription]] = None):
-        """
-        :param config:
-        :param batchSystem:
-        :param provisioner:
-        :param jobStore:
-        :param rootJob: Root job of the workflow that the leader will run
+    Represents the Toil leader.
 
+    Responsible for determining what jobs are ready to be scheduled, by
+    consulting the job store, and issuing them in the batch system.
+    """
+    def __init__(self,
+                 config: Config,
+                 batchSystem: AbstractBatchSystem,
+                 provisioner: Optional[AbstractProvisioner],
+                 jobStore: AbstractJobStore,
+                 rootJob: JobDescription,
+                 jobCache: Optional[Dict[Union[str, TemporaryID], JobDescription]] = None) -> None:
+        """
         If jobCache is passed, it must be a dict from job ID to pre-existing
         JobDescription objects. Jobs will be loaded from the cache (which can be
         downloaded from the jobStore in a batch) during the construction of the ToilState object.
+
+        :param config:      A Config object holding the original user options/settings/args.
+        :param batchSystem: The Batch System object containing functions to communicate with the
+                            specific batch system (run jobs, check jobs, etc.).
+        :param provisioner: Only for environments where we need to provision our own compute
+                            resources prior to launching jobs, for example, spin up an AWS instance.
+                            This is a Provisioner object with functions to create/remove resources.
+        :param jobStore:    The Jobstore object for storage, containing functions to talk to a
+                            central location where we store all of our files (jobs, inputs, etc.).
+        :param rootJob:     The first job of the workflow that the leader will run.
         """
         # Object containing parameters for the run
         self.config = config
@@ -154,7 +171,7 @@ class Leader:
         # this is used limit the number of services issued to the batch system
         self.serviceJobsIssued = 0
         self.serviceJobsToBeIssued: List[str] = [] # A queue of IDs of service jobs that await scheduling
-        #Equivalents for service jobs to be run on preemptible nodes
+        # Equivalents for service jobs to be run on preemptible nodes
         self.preemptableServiceJobsIssued = 0
         self.preemptableServiceJobsToBeIssued: List[str] = []
 
@@ -187,7 +204,7 @@ class Leader:
 
         # A dashboard that runs on the leader node in AWS clusters to track the state
         # of the cluster
-        self.toilMetrics = None
+        self.toilMetrics: Optional[ToilMetrics] = None
 
         # internal jobs we should not expose at top level debugging
         self.debugJobNames = ("CWLJob", "CWLWorkflow", "CWLScatter", "CWLGather",
@@ -197,6 +214,9 @@ class Leader:
 
         self.statusThrottler = LocalThrottle(self.config.statusWait)
 
+        # Control how often to poll the job store to see if we have been killed
+        self.kill_throttler = LocalThrottle(self.config.kill_polling_interval)
+
         # For fancy console UI, we use an Enlighten counter that displays running / queued jobs
         # This gets filled in in run() and updated periodically.
         self.progress_overall = None
@@ -215,15 +235,15 @@ class Leader:
         # report to its caller.
         self.recommended_fail_exit_code = 1
 
-    def run(self):
+    def run(self) -> Any:
         """
         This runs the leader process to issue and manage jobs.
 
         :raises: toil.leader.FailedJobsException if failed jobs remain after running.
 
         :return: The return value of the root job's run function.
-        :rtype: Any
         """
+        self.jobStore.write_kill_flag(kill=False)
 
         with enlighten.get_manager(stream=sys.stderr, enabled=not self.config.disableProgress) as manager:
             # Set up the fancy console UI if desirable
@@ -741,6 +761,11 @@ class Leader:
                 # Time to tell the user how things are going
                 self._reportWorkflowStatus()
 
+            if self.kill_throttler.throttle(wait=False):
+                if self.jobStore.read_kill_flag():
+                    logger.warning("Received kill via job store. Shutting down.")
+                    raise KeyboardInterrupt("killed via job store")
+
             # Make sure to keep elapsed time and ETA up to date even when no jobs come in
             self.progress_overall.update(incr=0)
 
@@ -947,13 +972,13 @@ class Leader:
         # batch system, it will show up here as waiting to run.
         return f"{running_job_count} jobs are running, {issued_job_count - running_job_count} jobs are issued and waiting to run"
 
-    def _reportWorkflowStatus(self):
+    def _reportWorkflowStatus(self) -> None:
         """
         Report the current status of the workflow to the user.
         """
 
         # For now just log our scheduling status message to the log.
-        # TODO: make this update fast enought to put it in the progress
+        # TODO: make this update fast enough to put it in the progress
         # bar/status line.
         logger.info(self._getStatusHint())
 
@@ -1107,18 +1132,18 @@ class Leader:
                 self.toilState.reset_job(jobStoreID)
                 replacementJob = self.toilState.get_job(jobStoreID)
             except NoSuchJobException:
-                # Avoid importing AWSJobStore as the corresponding extra might be missing
-                if self.jobStore.__class__.__name__ == 'AWSJobStore':
-                    # We have a ghost job - the job has been deleted but a stale read from
-                    # SDB gave us a false positive when we checked for its existence.
-                    # Process the job from here as any other job removed from the job store.
-                    # This is a temporary work around until https://github.com/BD2KGenomics/toil/issues/1091
-                    # is completed
-                    logger.warning('Got a stale read from SDB for job %s', issuedJob)
-                    self.processRemovedJob(issuedJob, result_status)
-                    return
-                else:
-                    raise
+                # We have a ghost job - the job has been deleted but a stale
+                # read from e.g. a non-POSIX-compliant filesystem gave us a
+                # false positive when we checked for its existence. Process the
+                # job from here as any other job removed from the job store.
+                # This is a hack until we can figure out how to actually always
+                # have a strongly-consistent communications channel. See
+                # https://github.com/BD2KGenomics/toil/issues/1091
+                logger.warning('Got a stale read for job %s; caught its '
+                'completion in time, but other jobs may try to run twice! Fix '
+                'the consistency of your job store storage!', issuedJob)
+                self.processRemovedJob(issuedJob, result_status)
+                return
             if replacementJob.logJobStoreFileID is not None:
                 with replacementJob.getLogFileHandle(self.jobStore) as logFileStream:
                     # more memory efficient than read().striplines() while leaving off the
