@@ -20,6 +20,7 @@ import re
 import shutil
 import signal
 import subprocess
+import sys
 import tempfile
 import threading
 import time
@@ -48,13 +49,16 @@ from urllib.error import HTTPError, URLError
 from urllib.request import urlopen
 
 import pytz
-from typing_extensions import Literal
+if sys.version_info >= (3, 8):
+    from typing import Literal
+else:
+    from typing_extensions import Literal
 
 from toil import ApplianceImageNotFound, applianceSelf, toilPackageDirPath
 from toil.lib.iterables import concat
 from toil.lib.memoize import memoize
 from toil.lib.threading import ExceptionalThread, cpu_count
-from toil.provisioners.aws import running_on_ec2
+from toil.lib.aws import running_on_ec2
 from toil.version import distVersion
 
 logger = logging.getLogger(__name__)
@@ -322,6 +326,17 @@ def get_temp_file(suffix: str = "", rootDir: Optional[str] = None) -> str:
         os.chmod(tmp_file, 0o777)  # Ensure everyone has access to the file.
         return tmp_file
 
+def needs_env_var(var_name: str, comment: Optional[str] = None) -> Callable[[MT], MT]:
+    """
+    Use as a decorator before test classes or methods to run only if the given
+    environment variable is set.
+    Can include a comment saying what the variable should be set to.
+    """
+    def decorator(test_item: MT) -> MT:
+        if not os.getenv(var_name):
+            return unittest.skip(f"Set {var_name}{' to ' + comment if comment else ''} to include this test.")(test_item)
+        return test_item
+    return decorator
 
 def needs_rsync3(test_item: MT) -> MT:
     """
@@ -364,23 +379,48 @@ def needs_aws_ec2(test_item: MT) -> MT:
     """Use as a decorator before test classes or methods to run only if AWS EC2 is usable."""
     # Assume we need S3 as well as EC2
     test_item = _mark_test('aws-ec2', needs_aws_s3(test_item))
-    if not os.getenv('TOIL_AWS_KEYNAME'):
-        # In addition to S3 we also need an SSH key to deploy with.
-        # TODO: We assume that if this is set we have EC2 access.
-        return unittest.skip("Set TOIL_AWS_KEYNAME to an AWS-stored SSH key to include this test.")(test_item)
+    # In addition to S3 we also need an SSH key to deploy with.
+    # TODO: We assume that if this is set we have EC2 access.
+    test_item = needs_env_var('TOIL_AWS_KEYNAME', 'an AWS-stored SSH key')(test_item)
+    return test_item
+
+
+def needs_aws_batch(test_item: MT) -> MT:
+    """
+    Use as a decorator before test classes or methods to run only if AWS Batch
+    is usable.
+    """
+    # Assume we need S3 as well as Batch
+    test_item = _mark_test('aws-batch', needs_aws_s3(test_item))
+    # Assume we have Batch if the user has set these variables.
+    test_item = needs_env_var('TOIL_AWS_BATCH_QUEUE', 'an AWS Batch queue name or ARN')(test_item)
+    test_item = needs_env_var('TOIL_AWS_BATCH_JOB_ROLE_ARN', 'an IAM role ARN that grants S3 and SDB access')(test_item)
+    try:
+        from toil.lib.aws import get_current_aws_region
+        if get_current_aws_region() is None:
+            # We don't know a region so we need one set.
+            # TODO: It always won't be set if we get here.
+            test_item = needs_env_var('TOIL_AWS_REGION', 'an AWS region to use with AWS batch')(test_item)
+
+    except ImportError:
+        return unittest.skip("Install Toil with the 'aws' extra to include this test.")(
+            test_item
+        )
     return test_item
 
 
 def needs_google(test_item: MT) -> MT:
-    """Use as a decorator before test classes or methods to run only if Google is usable."""
+    """
+    Use as a decorator before test classes or methods to run only if Google
+    Cloud is usable.
+    """
     test_item = _mark_test('google', test_item)
     try:
         from google.cloud import storage  # noqa
     except ImportError:
         return unittest.skip("Install Toil with the 'google' extra to include this test.")(test_item)
 
-    if not os.getenv('TOIL_GOOGLE_PROJECTID'):
-        return unittest.skip("Set TOIL_GOOGLE_PROJECTID to include this test.")(test_item)
+    test_item = needs_env_var('TOIL_GOOGLE_PROJECTID', "a Google project ID")(test_item)
     return test_item
 
 
@@ -425,9 +465,18 @@ def needs_tes(test_item: MT) -> MT:
     return test_item
 
 
-def needs_kubernetes(test_item: MT) -> MT:
+def needs_kubernetes_installed(test_item: MT) -> MT:
     """Use as a decorator before test classes or methods to run only if Kubernetes is installed."""
     test_item = _mark_test('kubernetes', test_item)
+    try:
+        import kubernetes
+    except ImportError:
+        return unittest.skip("Install Toil with the 'kubernetes' extra to include this test.")(test_item)
+    return test_item
+
+def needs_kubernetes(test_item: MT) -> MT:
+    """Use as a decorator before test classes or methods to run only if Kubernetes is installed and configured."""
+    test_item = needs_kubernetes_installed(test_item)
     try:
         import kubernetes
         try:
@@ -439,7 +488,8 @@ def needs_kubernetes(test_item: MT) -> MT:
                 return unittest.skip("Configure Kubernetes (~/.kube/config, $KUBECONFIG, "
                                      "or current pod) to include this test.")(test_item)
     except ImportError:
-        return unittest.skip("Install Toil with the 'kubernetes' extra to include this test.")(test_item)
+        # We should already be skipping this test
+        pass
     return test_item
 
 
@@ -568,6 +618,32 @@ def needs_server(test_item: MT) -> MT:
     else:
         return test_item
 
+def needs_celery_broker(test_item: MT) -> MT:
+    """
+    Use as a decorator before test classes or methods to run only if RabbitMQ is set up to take Celery jobs.
+    """
+    test_item = _mark_test('celery', test_item)
+    test_item = needs_env_var('TOIL_WES_BROKER_URL', "a URL to a RabbitMQ broker for Celery")(test_item)
+    return test_item
+
+def needs_wes_server(test_item: MT) -> MT:
+    """
+    Use as a decorator before test classes or methods to run only if a WES
+    server is available to run against.
+    """
+    test_item = _mark_test('wes_server', test_item)
+
+    wes_url = os.environ.get('TOIL_WES_ENDPOINT')
+    if not wes_url:
+        return unittest.skip(f"Set TOIL_WES_ENDPOINT to include this test")(test_item)
+
+    try:
+        urlopen(f"{wes_url}/ga4gh/wes/v1/service-info")
+    except (HTTPError, URLError) as e:
+        return unittest.skip(f"Run a WES server on {wes_url} to include this test")(test_item)
+
+    return test_item
+
 
 def needs_local_appliance(test_item: MT) -> MT:
     """
