@@ -19,7 +19,7 @@ from toil.lib.aws.ami import (
     get_flatcar_ami,
     official_flatcar_ami_release,
 )
-from toil.lib.ec2 import establish_boto3_session
+from toil.lib.aws.session import establish_boto3_session
 from toil.test import ToilTest, needs_aws_ec2
 
 logger = logging.getLogger(__name__)
@@ -54,3 +54,11 @@ class AMITest(ToilTest):
             ami = aws_marketplace_flatcar_ami_search(self.ec2_client)
             self.assertEqual(len(ami), len('ami-02b46c73fed689d1c'))
             self.assertTrue(ami.startswith('ami-'))
+
+        with self.subTest('Test flatcar AMI finder architecture parameter.'):
+            amis = set()
+            for arch in ['amd64', 'arm64']:
+                ami = get_flatcar_ami(self.ec2_client, architecture=arch)
+                self.assertTrue(ami.startswith('ami-'))
+                amis.add(ami)
+            self.assertTrue(len(amis) == 2)
