@@ -44,7 +44,7 @@ def main() -> None:
                         " On Google/GCE, this is the ssh key pair.")
     parser.add_argument("--owner", dest='owner',
                         help="The owner tag for all instances. If not given, the value in"
-                        " --keyPairName will be used if given.")
+                        "TOIL_OWNER_TAG will be used, or else the value of --keyPairName.")
     parser.add_argument("--boto", dest='botoPath',
                         help="The path to the boto credentials directory. This is transferred "
                         "to all nodes in order to access the AWS jobStore from non-AWS instances.")
@@ -140,7 +140,7 @@ def main() -> None:
                     # Provision fixed nodes
                     nodeCounts.append(int(spec))
 
-    owner = options.owner or options.keyPairName or 'toil'
+    owner = options.owner or os.getenv('TOIL_OWNER_TAG') or options.keyPairName or 'toil'
 
     # Check to see if the user specified a zone. If not, see if one is stored in an environment variable.
     options.zone = options.zone or os.environ.get(f'TOIL_{options.provisioner.upper()}_ZONE')
